#include <aduc7023.h>

//*** <<< Use Configuration Wizard in Context Menu >>> *** 
//  

/*********************************************************************
  All ADuC7023 library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.

 Author        : ADI - Apps            www.analog.com/ADuC7023
 Date:        26/02/2009
 Description: Demonstrates the Nested Vectored Interrupt Controller
*********************************************************************/

// Constants
#define SW_INT                 1  
#define T0_INT                 2  
#define T1_INT                 3  
#define T2_INT                 4  
#define FLASH_INT              5  
#define ADC_INT                6 
#define PLL_INT                7 
#define I2C0M_INT              8 
#define I2C0S_INT              9 
#define I2C1M_INT              10 
#define I2C1S_INT              11 
#define SPI_INT                12 
#define XIRQ0_INT              13 
#define COMP_INT               14 
#define PSM_INT                15
#define XIRQ1_INT              16 
#define PLA0_INT               17 
#define XIRQ2_INT              18 
#define XIRQ3_INT              19 
#define PLA1_INT               20 
#define PWM_INT                21

#define IRQ3CLRI_BIT           19 
#define IRQ2CLRI_BIT           18 
#define IRQ1CLRI_BIT           16 
#define IRQ0CLRI_BIT           13 

// IRQCONN Bit settings
#define IRQCONN_IRQ_NEST_BIT      0
#define IRQCONN_FIQ_NEST_BIT      1

// IRQP0 MMR Bit Designations
#define IRQP0_PLLPI_BIT           28
#define IRQP0_ADCPI_BIT           24
#define IRQP0_FPI_BIT             20
#define IRQP0_T2PI_BIT            16
#define IRQP0_T1PI_BIT            12
#define IRQP0_T0PI_BIT            8
#define IRQP0_SWIINTP_BIT         4

// IRQP1 MMR Bit Designations
#define IRQP1_PSMPI_BIT           28
#define IRQP1_COMPPI_BIT          24
#define IRQP1_IRQ0PI_BIT          20
#define IRQP1_SPIMPI_BIT          16
#define IRQP1_I2C1SPI_BIT         12
#define IRQP1_I2C1MPI_BIT         8
#define IRQP1_I2C0SPI_BIT         4
#define IRQP1_I2C0MPI_BIT         0


// IRQP2 MMR Bit Designations
#define IRQP2_PWMPI_BIT           20
#define IRQP2_PLA1PI_BIT          16
#define IRQP2_IRQ3PI_BIT          12
#define IRQP2_IRQ2PI_BIT          8
#define IRQP2_PLA0PI_BIT          4
#define IRQP2_IRQ1PI_BIT          0


#define DIR_VAL                  0x80000000


/* Local Function Prototypes */

/* FIQ\IRQ Handler Functions */
void IRQ0_FIQ_SOURCE (void);
void IRQ1_SW_INT     (void);
void IRQ2_T0_INT     (void);
void IRQ3_T1_INT     (void);  
void IRQ4_T2_INT     (void);
void IRQ5_Flash_INT  (void);
void IRQ6_ADC_INT    (void);
void IRQ7_PLL_INT    (void);
void IRQ8_I2C0M_INT  (void);
void IRQ9_I2C0S_INT  (void);
void IRQ10_I2C1M_INT (void);
void IRQ11_I2C1S_INT (void);
void IRQ12_SPIM_INT  (void);
void IRQ13_XIRQ0_INT (void);
void IRQ14_COMP_INT  (void);
void IRQ15_PSM_INT   (void);
void IRQ16_XIRQ1_INT (void);
void IRQ17_PLA0_INT  (void);
void IRQ18_XIRQ2_INT (void);
void IRQ19_XIRQ3_INT (void);
void IRQ20_PLA1_INT  (void);
void IRQ21_PWM_INT   (void);
void IRQx_NULL_INT   (void);


/* Exception Handlers */
void PAB_Function    (void);
void DAB_Function    (void);
void SWI_Function    (void);
void UNDEF_Function  (void);

/*    
   Define a type of function suitable for handling interrupts (and for
   inclusion in our table of interrupts)

   Requirements
   1) Does not accept paramters
   2) Does not return any value
   3) Will return normally to its caller
*/
typedef void (*tyIntHndlr)(void);  


/*    Table of Interupts Handlers 

      The VIC can handle up to 32 interrupt sources but ~20 are implemented 
      on the ADuC7060.

      The table MUST be aligned to a 128 byte boundary.
*/
const tyIntHndlr IntHndlrTable[32] __attribute__ ((aligned (128)))  = 
{
   IRQ0_FIQ_SOURCE,             /* 0  - Any FIQ active         */
   IRQ1_SW_INT    ,             /* 1  - Software               */
   IRQ2_T0_INT    ,             /* 2  - Timer 0                */
   IRQ3_T1_INT    ,             /* 3  - Timer 1                */
   IRQ4_T2_INT    ,             /* 4  - Timer 2                */
   IRQ5_Flash_INT ,             /* 5  - Flash                  */
   IRQ6_ADC_INT   ,             /* 6  - ADC		               */
   IRQ7_PLL_INT   ,             /* 7  - PLL                    */
   IRQ8_I2C0M_INT ,             /* 8  - I2C0 Master            */
   IRQ9_I2C0S_INT ,             /* 9  - I2C0 Slave             */
   IRQ10_I2C1M_INT ,            /* 10 - I2C1 Master            */
   IRQ11_I2C1S_INT ,            /* 11 - I2C1 Slave 		       */
   IRQ12_SPIM_INT  ,            /* 12 - SPI Master interrupt   */
   IRQ13_XIRQ0_INT,             /* 13 - External interrupt 0   */
   IRQ14_COMP_INT,              /* 14 - Comparator interrupt   */
   IRQ15_PSM_INT  ,             /* 15 - PSM interrupt 		   */
   IRQ16_XIRQ1_INT ,            /* 16 - External interrupt 1   */
   IRQ17_PLA0_INT ,             /* 17 - PLA0 interrupt 		   */
   IRQ18_XIRQ2_INT,             /* 18 - External interrupt 2   */
   IRQ19_XIRQ3_INT,             /* 19 - External interrupt 3   */
   IRQ20_PLA1_INT ,             /* 20 - PLA1 interrupt         */
   IRQ21_PWM_INT  ,             /* 21 - PWM  interrupt         */
   IRQx_NULL_INT  ,             /* 22 - Unused                 */
   IRQx_NULL_INT  ,             /* 23 - Unused                 */
   IRQx_NULL_INT  ,             /* 24 - Unused                 */
   IRQx_NULL_INT  ,             /* 25 - Unused                 */
   IRQx_NULL_INT  ,             /* 26 - Unused                 */
   IRQx_NULL_INT  ,             /* 27 - Unused                 */
   IRQx_NULL_INT  ,             /* 28 - Unused                 */
   IRQx_NULL_INT  ,             /* 29 - Unused                 */
   IRQx_NULL_INT  ,             /* 30 - Unused                 */
   IRQx_NULL_INT  ,             /* 31 - Unused                 */
};

static unsigned long ulLoop   = 0x0;
static unsigned short ADCDATA = 0;
// Function    : Delay
// Author      : ADI
// Description : main
// Revision    : 26.02.2009 - Initial
void Delay(void)
{
   volatile int iDelay = 0x0FFFF;

   while (iDelay--);
}
void ADCpoweron(int time)
{
	ADCCON = 0x20;	 					// power-on the ADC
	while (time >=0)	  				// wait for ADC to be fully powered on
    time--;
}
// Function    : LedToggle
// Author      : ADI
// Description : Toggle the led
// Revision    : 24.02.2009 - Initial
void  LedToggle(void)
{

   // Apps EvalBoard with P0.7 to blink
   if (ulLoop%2)
     GP0SET = 0x00800000;
   else
     GP0CLR = 0x00800000;

   ulLoop++;
}

// Function    : main
// Author      : ADI
// Description : main
// Revision    : 26.02.2009 - Initial
int main(void)
{  
   unsigned long ulA,ulB, ulC;

   GP1CON = 0x0;
   GP0DAT = DIR_VAL;

	POWKEY1 = 0x01;					// Configure CPU Clock for 41.78MHz, CD=0
 	POWCON = 0x00;
 	POWKEY2 = 0xF4;	

   // Setup Exception handlers
   PABORT = PAB_Function;
   DABORT = DAB_Function;
   SWI    = SWI_Function;
   UNDEF  = UNDEF_Function;


   // Let the VIC know about our vector table
   IRQBASE           = (unsigned long)(((unsigned long)IntHndlrTable) >> 7);

   // Enable nesting and prioritisation of IRQ and FIQ interrupts
   IRQCONN           = (0x1UL << IRQCONN_IRQ_NEST_BIT) |
                       (0x1UL << IRQCONN_FIQ_NEST_BIT) ;

   // Set priorities
   IRQP0             = (0x3UL << IRQP0_PLLPI_BIT)  | 
                       (0x6UL << IRQP0_ADCPI_BIT)  |
   					   (0x3UL << IRQP0_FPI_BIT)    | 
                       (0x1UL << IRQP0_T2PI_BIT)   |
                       (0x2UL << IRQP0_T1PI_BIT)   |
                       (0x2UL << IRQP0_T0PI_BIT)   |
                       (0x0UL << IRQP0_SWIINTP_BIT) ;

   IRQP1             = (0x0UL << IRQP1_PSMPI_BIT)  | 
   					   (0x0UL << IRQP1_COMPPI_BIT) |
                       (0x1UL << IRQP1_IRQ0PI_BIT) |
                       (0x0UL << IRQP1_SPIMPI_BIT) |
                       (0x1UL << IRQP1_I2C1SPI_BIT)|
                       (0x0UL << IRQP1_I2C1MPI_BIT)|
                       (0x0UL << IRQP1_I2C0SPI_BIT)|
                       (0x0UL << IRQP1_I2C0MPI_BIT );

   IRQP2             = (0x0UL << IRQP2_PWMPI_BIT)  | 
   					   (0x0UL << IRQP2_PLA1PI_BIT) |
                       (0x0UL << IRQP2_IRQ3PI_BIT) |
                       (0x0UL << IRQP2_IRQ2PI_BIT) |
                       (0x0UL << IRQP2_PLA0PI_BIT) |
                       (0x0UL << IRQP2_IRQ1PI_BIT) ;

   // Configure ADC
	ADCpoweron(20000);				// power on ADC
	ADCCP = 0x00;	  				// select ADC channel 0
	REFCON = 0x01;					// internal 2.5V reference. 2.5V on Vref pin
	ADCCON = 0xEE4;					// ADC Config: fADC/8, acq. time = 8 clocks 

   // Configure external interrupts
   IRQCONE           = 0x03;        // XIRQ0 low edge triggered
   IRQCONE          |= 0x0C;        // XIRQ1 low edge triggered
   IRQCONE          |= 0x30;        // XIRQ2 low edge triggered
   IRQCONE          |= 0xC0;        // XIRQ3 low edge triggered
   // Configure Timer 0
   T0LD            = 0x300;       // Enable Timer 1
   T0CON           = 0x4C0;       // Periodic, HCLK, div/1


   // Configure Timer 1
   T1LD            = 0x700;       // Enable Timer 1
   T1CON           = 0x4C0;       // Periodic, HCLK, div/1

   


   // Enable IRQs
   IRQEN           = (1UL << T1_INT)  |
                     (1UL << ADC_INT) |
				//	 (1UL << XIRQ3_INT)|
				//	 (1UL << XIRQ2_INT)|
				//	 (1UL << XIRQ1_INT)|
                     (1UL << XIRQ0_INT);

   // Enable FIQs
   FIQEN           =    (1UL << T0_INT)  | 
                       (1UL << PWM_INT); 

   ulA = 0x3;
   ulB = 0x4;
   while (1)
     {
     ulC = ulA+ulB+ulC;
     // Wait for interrupts
     LedToggle();
     Delay();
     }
}


// Function    : IRQ0_FIQ_SOURCE
// Author      : ADI
// Description : Interrupt handler
// Revision    : 24.02.2009 - Initial
void IRQ0_FIQ_SOURCE (void)
{
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : IRQ1_SW_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQ1_SW_INT     (void)
{
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : IRQ2_T0_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009  - Initial
void IRQ2_T0_INT     (void)
{
   T0CLRI = 0xFF;

}

// Function    : IRQ3_T1_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQ3_T1_INT     (void)  
{
   // Clear the interrupt
   T1CLRI = 0xFF;
 //   while (1) {				 // Uncomment these lines to prove IRQ nesting
 //  	GP0DAT ^= 0x800000;	}

}

// Function    : IRQ4_T2_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQ4_T2_INT     (void)
{
   // Clear the interrupt
   T2CLRI = 0xFF;
}
// Function    : IRQ5_Flash_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQ5_Flash_INT     (void)
{
   // Clear the interrupt
   
}
// Function    : IRQ6_ADC_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQ6_ADC_INT   (void)
{
   // Clear the interrupt
   
	ADCDATA = (ADCDAT >> 16);
 
}
void IRQ7_PLL_INT   (void)
{

 
}
void IRQ8_I2C0M_INT   (void)
{

 
}
void IRQ9_I2C0S_INT   (void)
{

 
}
void IRQ10_I2C1M_INT   (void)
{

 
}
void IRQ11_I2C1S_INT   (void)
{

 
}


// Function    : IRQ12_SPIM_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQ12_SPIM_INT  (void)
{
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : IRQ13_XIRQ0_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQ13_XIRQ0_INT (void)
{
   // Clear an edge level interrupt
   IRQCLRE = 0x1UL << IRQ0CLRI_BIT;
}
void IRQ14_COMP_INT (void)
{
   
}
void IRQ15_PSM_INT (void)
{
   
}
// Function    : IRQ16_XIRQ1_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQ16_XIRQ1_INT (void)
{
   // Clear an edge level interrupt
   IRQCLRE = 0x1UL << IRQ1CLRI_BIT;
}

void IRQ17_PLA0_INT (void)
{
   
}



// Function    : IRQ18_XIRQ2_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQ18_XIRQ2_INT (void)
{
   // Clear an edge level interrupt
   IRQCLRE = 0x1UL << IRQ2CLRI_BIT;
}

// Function    : IRQ19_XIRQ3_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQ19_XIRQ3_INT (void)
{
   // Clear an edge level interrupt
   IRQCLRE = 0x1UL << IRQ3CLRI_BIT;
}
void IRQ20_PLA1_INT (void)
{

}
void IRQ21_PWM_INT (void)
{

}

// Function    : IRQx_NULL_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.02.2009 - Initial
void IRQx_NULL_INT   (void)
{
   // This should never be called
   while(1); // Infinite loop until we provide an actual handler
}


// Function    : PAB_Function
// Author      : ADI
// Description : Program abort exception handler
// Revision    : 26.02.2009 - Initial
void PAB_Function(void)
{
   // This should never be called
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : DAB_Function
// Author      : ADI
// Description : Data abort exception handler
// Revision    : 26.02.2009 - Initial
void DAB_Function(void)
{
   // This should never be called
   while(1); // Infinite loop until we provide an actual handler
}


// Function    : SWI_Function
// Author      : ADI
// Description : SWI exception handler
// Revision    : 26.02.2009 - Initial
void SWI_Function(void)
{
   // This should never be called
   while(1); // Infinite loop until we provide an actual handler
}


// Function    : UNDEF_Function
// Author      : ADI
// Description : UNDEF exception handler
// Revision    : 26.02.2009 - Initial
void UNDEF_Function(void)
{
   // This should never be called
   while(1); // Infinite loop until we provide an actual handler
}




